﻿
using System;
using System.Windows.Forms;

/* To solve a matrix equation by Gram-Schmidt QR decomposition   */
/*                    www.mosismath.com                          */

namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        const int MaxOrder = 4;
        int actOrder = MaxOrder;
        double[,] q = new double[MaxOrder, MaxOrder];
        double[,] r = new double[MaxOrder, MaxOrder];

        double[,] a = new double[MaxOrder, MaxOrder];
        double[] y = new double[MaxOrder];
        double[] x = new double[MaxOrder];
 

        double InnerProd(double[,] a, double[,] b, int colA, int colB, int order)
        {
            int i;
            double c = 0;
            for (i = 0; i < order; i++)
            {
                c = c + a[i, colA] * b[i, colB];
            }
            return c;
        }


        public double[] MultMatrixTransVect(double[,] tr, double[] b)
        {
            int j, k;
            double[] al = new double[b.Length];
            double[] yl = new double[b.Length];

            for (j = 0; j < b.Length; j++)
            {
                al[j] = 0.0;
                for (k = 0; k < b.Length; k++)
                {
                    al[j] = al[j] + (tr[k, j] * b[k]);
                }
            }
            return al;
        }


        double Len(double[] a)
        {
            int i;
            double len = 0.0;
            for (i = 0; i < a.Length; i++)
            {
                len = len + a[i] * a[i];
            }
            len = Math.Sqrt(len);
            return len;
        }


        public void CalcGranSchmidt(int order)
        {
            int i, k;
            int actCol;
            double len = 0.0;
            double[] v = new double[order];
            for (i = 0; i < order; i++)
            {
                len = len + a[i, 0] * a[i, 0];
            }
            len = Math.Sqrt(len);

            // q1 is different
            for (i = 0; i < order; i++)
            {
                q[i, 0] = a[i, 0] / len;
            }

            //r11 can be calculated right away
            r[0, 0] = InnerProd(a, a, 0, 0, order)/len;

            for (actCol = 1; actCol < order; actCol++)
            {
                for (i = 0; i < order; i++)
                {
                    v[i] = a[i, actCol];
                    q[i, actCol] = 0.0;
                }

                for (i = 0; i < actCol; i++)
                {
                    // calculate the elements of R
                    r[i, actCol] = InnerProd(q, a, i, actCol, order);

                    // calculate the temporary V vector
                    for (k = 0; k < order; k++)
                    {
                        v[k] = v[k] - q[k, i] * r[i, actCol];
                    }
                }

                // the length of V 
                len = Len(v);

                // one column od Q is V / lenght of V
                for (i = 0; i < order; i++)
                {
                    q[i, actCol] = v[i] / len;
                }

                // the diagonal elements of R must be calculated separately
                r[actCol, actCol] = InnerProd(q, a, actCol, actCol, order);
            }
        }


        public void Solve()
        {
            int k, l;
            for (k = MaxOrder - 1; k >= 0; k--)
            {
                for (l = MaxOrder - 1; l > k; l--)
                {
                    y[k] = y[k] - x[l] * r[k, l];
                }
                x[k] = y[k] / r[k, k];
            }
        }


        public MainWin()
        {
            int i, k;
            DataGridViewCell cell;
           
            InitializeComponent();

            GMatrixA.ColumnCount = MaxOrder;
            GMatrixA.RowCount = MaxOrder;
            GVectorY.RowCount = MaxOrder;
            GVectorY.ColumnCount = 1;
            GVectorX.RowCount = 1;
            GVectorX.ColumnCount = MaxOrder;
            for (i = 0; i < GMatrixA.ColumnCount; i++)
                GMatrixA.Columns[i].Width = 160;


            a[1, 0] = 3.0;
            a[1, 1] = -5.0;
            a[1, 2] = 4.0;
            a[1, 3] = -3.0;

            y[1] = -2.5;

            a[0, 0] = 0.0;
            a[0, 1] = 0.0;
            a[0, 2] = -3.0;
            a[0, 3] = 2.0;

            y[0] = -4.1;
            
            a[2, 0] = 3.0;
            a[2, 1] = 2.0;
            a[2, 2] = 0.0;
            a[2, 3] = 4.0;

            y[2] = 14.8;

            a[3, 0] = 5.0;
            a[3, 1] = -2.0;
            a[3, 2] = 3.0;
            a[3, 3] = -3.0;

            y[3] = 4.0;

          

            for (i = 0; i < MaxOrder; i++)
            {
                for (k = 0; k < MaxOrder; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    cell.Value = Convert.ToString(a[k, i]);
                }
                cell = GVectorY[0,i];
                cell.Value = Convert.ToString(y[i]);
            }
        }


        private void button1_MouseClick(object sender, MouseEventArgs e)
        {
            int i, k;
            double[] buf = new double[actOrder];
            DataGridViewCell cell;
            
            for (i = 0; i < actOrder; i++)
            {
                for (k = 0; k < actOrder; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    a[k, i] = Convert.ToDouble(cell.Value);
                    cell = GVectorY[0, i];  // get cell to access
                    y[i] = Convert.ToDouble(cell.Value);
                    x[i] = 0;
                    r[k, i] = 0.0;
                    q[k, i] = 0.0;
                }
            }

            CalcGranSchmidt(actOrder);
            y = MultMatrixTransVect(q, y);
            Solve();

            for (i = 0; i < actOrder; i++)
            {
                for (k = 0; k < actOrder; k++)
                {
                    cell = GMatrixA[i, k];  // get cell to access
                    cell.Value = Convert.ToString(a[k, i]);
                }
                cell = GVectorX[i, 0];
                cell.Value = Convert.ToString(x[i]);
            }
        }
    }
}
